﻿// 作成 : 2006-2010 一歩二歩散歩

using System;
using System.Drawing;

using OpenTK.Graphics.OpenGL;
using gl = OpenTK.Graphics.OpenGL.GL;
using glu = OpenTK.Graphics.OpenGL.Glu;

namespace sample
{

    #region public static class Capture

    /// <summary>
    /// バッファの内容をキャプチャするためのクラス。
    /// </summary>
    public static class Capture
    {

        /// <summary>
        /// バッファの内容をキャプチャする(RGBA)。
        /// </summary>
        /// <param name="target">キャプチャするバッファを指定する</param>
        /// <param name="x">キャプチャする領域の左下隅のx座標</param>
        /// <param name="y">キャプチャする領域の左下隅のy座標</param>
        /// <param name="width">キャプチャする領域の幅</param>
        /// <param name="height">キャプチャする領域の高さ</param>
        /// <param name="disableAlpha">
        /// アルファ値を無効にするかどうかを指定する。
        /// trueにした場合、キャプチャした画像のアルファ値はすべて1になる。
        /// </param>
        /// <returns>キャプチャした画像</returns>
        public static Bitmap CaptureBufferAsBitmapRGBA( ReadBufferMode target, int x, int y, int width, int height, bool disableAlpha )
        {
            // Bitmap のフォーマット
            System.Drawing.Imaging.PixelFormat bmpPixelFormat
                = disableAlpha ? System.Drawing.Imaging.PixelFormat.Format24bppRgb : System.Drawing.Imaging.PixelFormat.Format32bppArgb;

            //読み取ったデータをこのBitmapに変換する。
            //まずはインスタンスの生成。
            Bitmap bmp = new Bitmap( width, height, bmpPixelFormat );

            //このbmpの画像データを直接操作するために、
            //BitmapDataというクラスを使う。
            System.Drawing.Imaging.BitmapData bmpData = bmp.LockBits( new Rectangle( 0, 0, bmp.Width, bmp.Height ),
                                                                      System.Drawing.Imaging.ImageLockMode.WriteOnly,
                                                                      bmpPixelFormat );

            //読み取るバッファを指定。
            gl.ReadBuffer( target );

            // 型名が紛らわしいが、glReadPixelsで使う方のピクセルフォーマット
            OpenTK.Graphics.OpenGL.PixelFormat glPixFormat
                = disableAlpha ? OpenTK.Graphics.OpenGL.PixelFormat.Bgr : PixelFormat.Bgra;

            //バッファから読み取る。
            gl.ReadPixels( x, y, width, height, glPixFormat, PixelType.UnsignedByte, bmpData.Scan0 );

            //LockBitsでロックしたものは、
            //UnlockBitsで、きちんと解放すること。
            bmp.UnlockBits( bmpData );

            //最後に、上下を反転させて仕上げる。
            bmp.RotateFlip( RotateFlipType.RotateNoneFlipY );

            //おしまい。

            return bmp;
        }

    }

    #endregion public static class Capture

}